// EOColumnAssociation.h
// Enterprise Objects Framework
// Copyright (c) 1993, NeXT Computer, Inc.  All rights reserved. 

#import "EOAssociation.h"
#import "EOController.h"
#import "NXTableView.h"

// The EOColumnAssociation object is the subclass of EOAssociation responsible
// for making an assocation between a column of a NXTableView and methods in
// the modelling objects for that column.  The EOColumnAssociations take over
// the column identifiers to point to themselves.  Furthermore, all of the
// EOColumnAssociations associated with a given NXTableView must be
// associations of the same EOController.

@class NSArray,EOController,NXTableView, NXTableVector;


@interface EOColumnAssociation : EOAssociation
{
    NXTableVector *vector;
    NXTableView *tableView;	// The TableView our column is in
    				// NOTE: NEVER use the destination
    				// ivar in a ColumnAssociation; it's
    				// always tableView instead.
    unsigned updating : 1;	// This flag indicated we're in the
    				// middle of some higher-level operation,
    				// and thus that any message from the TV
    				// should be ignored.
    unsigned _unused : 31;
    void *_values;
}

// Designated initializer for EOColumnAssociation
- initWithController:(EOController *)aController 
    key: (NSString *)aKey destination:(NXTableVector *)theTV;

- (NXTableView *)tableView;
- (void)setTableView:(NXTableView *)newTableView;

// These following methods should not be invoked directly.

// Methods used by NXTableView to ask for values (messages passed on to
// appropriate association objects).
- getValueFor:identifier at:(unsigned int) aPosition into:aValue;
- setValueFor:identifier at:(unsigned int) aPosition from:aValue;

// Methods that then actually get invoked to return the data (i.e., the above
// methods call these methods)
- getValueAt:(unsigned int) aPosition into:aValue;
- setValueAt:(unsigned int) aPosition from:aValue;


@end




