/*
**      NXEditableFormatter.h
**      Copyright (c) 1992, NeXT Computer, Inc.  All rights reserved. 
*/

#import "NXFormatter.h"
#import "NXTableProtocols.h"

#import <objc/Object.h>
#import <appkit/appkit.h>


// NXEditableFormatter is functionally identical to the Database Kit's
// DBEditableFormatter. Please refer to the DatabaseKit documentation for use
// of this class.


@interface NXEditableFormatter : NXFormatter
{
@private
	unsigned int	_efReserved[2];

	id		rowAttrs;
	id		columnAttrs;
	int		editRow;
	int		editColumn;

	/* NB: we don't have to swap these because they're not archived... */
	BOOL		isEditing:1,	/* is column handling a cell edit? */
			isModified:1,	/* is cell known to be modified? */
			useRowPos:1,	/* use row position or identifier */
			useColumnPos:1,	/* ditto for columns */
			delWill1:1,	/* methods delegate responds to */
			delWill2:1,
			delDid:1,
			delWillStatic1:1,
			delWillStatic2:1,
			delDidStatic:1;

@protected
	id		font;		/* font with which to draw */
	id		editView;	/* view we're currently editing in */
	id		drawCell;	/* one cell for drawing */
}


- init;
- free;

- font;
- setFont:aFont;

- setDelegate:newDelegate;

- drawFieldAt:(unsigned int) row :(unsigned int) column
	inside:(NXRect *) frame inView:view
	withAttributes:(id <NXTableVectors>) rowAttrs
		      :(id <NXTableVectors>) columnAttrs
	usePositions:(BOOL) useRowPos :(BOOL) useColumnPos;

- (BOOL) editFieldAt:(unsigned int) row :(unsigned int) column
	inside:(NXRect *) frame inView:view
	withAttributes:(id <NXTableVectors>) rowAttributes
		      :(id <NXTableVectors>) columnAttributes
	usePositions:(BOOL) useRowPos :(BOOL) useColumnPos
	onEvent:(NXEvent *) theEvent;

- abortEditing;
- (BOOL) endEditing;

- write:(NXTypedStream *) stream;
- read:(NXTypedStream *) stream;

@end


@interface Object(NXFormatterValidation)
- (BOOL) formatterWillChangeValueFor:identifier at:(unsigned int) position sender:sender;
- (BOOL) formatterWillChangeValueFor:identifier at:(unsigned int) position to:aValue sender:sender;
- formatterDidChangeValueFor:identifier at:(unsigned int) position to:aValue sender:sender;

- (BOOL) formatterWillChangeValueFor:rowIdentifier :columnIdentifier sender:sender;
- (BOOL) formatterWillChangeValueFor:rowIdentifier :columnIdentifier to:aValue sender:sender;
- formatterDidChangeValueFor:rowIdentifier :columnIdentifier to:aValue sender:sender;
@end


@protocol NXFormatterViewEditing
- (BOOL) formatterDidEndEditing:sender endChar:(unsigned short)whyEnd;
@end

