/*	NSScanner.h
	Simple scanning on strings
  	Copyright 1993, 1994, NeXT, Inc. All rights reserved.
*/
#import <foundation/NSString.h>
#import <foundation/NSCharacterSet.h>

/* Abstract class for scanning strings.
*/
@interface NSScanner : NSObject <NSCopying>

- (NSString *)string;
- (void)setScanLocation:(unsigned)pos;
- (unsigned)scanLocation;
- (void)setCharactersToBeSkipped:(NSCharacterSet *)set;	/* whitespaceAndNewlineCharacterSet by default */
- (void)setCaseSensitive:(BOOL)flag;			/* NO by default */

@end

@interface NSScanner (NSExtendedScanner)

- (NSCharacterSet *)charactersToBeSkipped;
- (BOOL)caseSensitive;

- (BOOL)scanInt:(int *)value;
- (BOOL)scanLongLong:(long long *)value;
- (BOOL)scanFloat:(float *)value;
- (BOOL)scanDouble:(double *)value;
- (BOOL)scanString:(NSString *)string intoString:(NSString **)value;
- (BOOL)scanCharactersFromSet:(NSCharacterSet *)set intoString:(NSString **)value;

/* If the next two don't find the desired string or characters, they parse to the end and return YES. They will return NO only if the no characters were parsed.
*/
- (BOOL)scanUpToString:(NSString *)string intoString:(NSString **)value;
- (BOOL)scanUpToCharactersFromSet:(NSCharacterSet *)set intoString:(NSString **)value;

/* Returns YES if only characters left are those in the charactersToBeSkipped set.
*/
- (BOOL)isAtEnd;

/* Creating scanners.
*/
- initWithString:(NSString *)string;
+ scannerWithString:(NSString *)string;

@end

